"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.forEachDeep = void 0;
/**
 * Iterates over elements of collection invoking iteratee for each element.
 * The iteratee is invoked with three arguments: (value, path).
 * path is an array containing keys of current value
 *
 * @param obj object to iterate over
 * @param iteratee The function invoked per iteration.
 */
function forEachDeep(obj, iteratee) {
    const helper = (obj, path) => {
        Object.entries(obj).forEach(([key, value]) => {
            iteratee(value, [...path, key]);
            if (typeof value === 'object' && value && !Array.isArray(value)) {
                return helper(value, [...path, key]);
            }
        });
    };
    helper(obj, []);
    return;
}
exports.forEachDeep = forEachDeep;
//# sourceMappingURL=for-each-deep.util.js.map